//
//  File.h
//  UsenetEngine
//
//  Created by Ger Teunis on 12/6/09.
//  Copyright (c) 2013 VortexApps. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "NNTPEnums.h"
@class ConnectionPoolItem;
@class yDecodedMessage;
@class FileArticle;
@class NZB;
@class PersistenceHandler;


@interface File : NSObject {
    
    int databaseId;
	NZB *parentNZB;
	NSString *path;
	NSString *fileName;
	NSMutableArray *fileArticleQueue;
	FileState state;
	NSLock *threadLock;
	NSFileHandle *fileHandle;
	NSArray *groups;
	long long fileSize;
    long long downloadedSize;
    long long totalDownloadSize;
    bool extractPasswordRequired;
    bool extractPasswordHasHintFiles;
    NSString* extractPassword;
    bool registeredObserver;
    NSLock *downloadSizeLock;
    NSLock *fileArticleQueueLock;
    NSMutableArray *articlesForBackupServer;
    bool isSmartCheckable;
    NSDate *postDate;
    NSString *fileHeader;
    NSNumber *crc32; //crc32 from sfv, not the calculated one
    bool crc32CheckFailed;
    NSNumber *isUUEncoded;
    int uuencodeSegmentSize;
    int cachedFileArticleQueueCount;
    int cachedParentNZBDatabaseId;
    bool persistenceShouldReportChanges;
    PersistenceHandler *persistenceHandler;
}

@property (nonatomic) int databaseId;
@property (nonatomic, retain) NSArray* groups;
@property (nonatomic, retain) NSString *path;
@property (nonatomic, retain) NSString *fileName;
@property (nonatomic) FileState state;
@property (nonatomic, retain) NSMutableArray *fileArticleQueue;
@property (nonatomic, retain) NSLock *threadLock;
@property (nonatomic, retain) NSFileHandle *fileHandle;
@property (nonatomic) long long fileSize;
@property (nonatomic) long long downloadedSize;
@property (nonatomic) long long totalDownloadSize;
@property (nonatomic, assign) NZB *parentNZB;
@property (nonatomic) bool extractPasswordRequired;
@property (nonatomic) bool extractPasswordHasHintFiles;
@property (nonatomic, retain) NSString* extractPassword;
@property (nonatomic) bool isSmartCheckable;
@property (nonatomic, retain) NSDate *postDate;
@property (nonatomic, retain) NSNumber *crc32;
@property (nonatomic) bool crc32CheckFailed;
@property (nonatomic, retain) NSNumber *isUUEncoded;
@property (nonatomic) int uuencodeSegmentSize;
@property (nonatomic) int cachedFileArticleQueueCount;
@property (nonatomic) int cachedParentNZBDatabaseId;
@property (nonatomic, assign) bool persistenceShouldReportChanges;

#pragma mark -
#pragma mark Sizes and states
-(void)processMissingNZBArticles;
-(void)sortArticleQueue;
-(bool)fileArticleQueueLoaded;
-(unsigned long long)calculateDownloadedSize;
-(unsigned long long)calculateTotalDownloadSize;
-(float)getDownloadedPercentage;
-(bool)smartCheckable;
-(void)setAllArticlesToState:(FileArticleState)newState;
-(void)addDownloadedSize:(int)downloaded;
-(void)addArticleToBackupQueue:(FileArticle*)article;
-(void)removeArticleFromBackupQueue:(FileArticle*)article;
-(void)checkBackupServerArticlesValidity;

#pragma mark -
#pragma mark Rest
-(void)checkIsPasswordProtected;
-(void)moveFinalToPartial;
-(NSString*)getFileHeader;
-(bool)hasArticlesInBackupQueue;
-(bool)retryDownloadMissingParts;
-(bool)closeFile;
-(NSFileHandle*)openFile:(NSString*)filePath;
-(int)downloadArticle:(ConnectionPoolItem*)connection isFinalResult:(BOOL*)finalResult;
-(long long)saveFile:(long long)maxSize;
-(int)saveArticle:(FileArticle*)article;
-initWithNZB:(NZB*)ParentNZB DatabaseId:(int)DatabaseId Path:(NSString*)Path Filename:(NSString*)FileName PostDate:(NSDate*)newPostDate Groups:(NSArray*)fileGroups;
-(void)dealloc;

@end
